library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity tupacvhdl is
  
  port (
    signal CPU_RESET, CLOCK_50 : in std_logic;                -- 50 MHz clock
    VGA_CLK,                                            -- Clock
    VGA_HS,                                             -- H_SYNC
    VGA_VS,                                             -- V_SYNC
    VGA_BLANK,                                          -- BLANK
    VGA_SYNC : out std_logic;                           -- SYNC
    VGA_R,                                              -- Red[9:0]
    VGA_G,                                              -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0);           -- Blue[9:0]


    SRAM_DQ : inout std_logic_vector(15 downto 0); -- Data bus 16 Bits
    SRAM_ADDR : out std_logic_vector(17 downto 0); -- Address bus 18 Bits
    SRAM_UB_N,                                     -- High-byte Data Mask
    SRAM_LB_N,                                     -- Low-byte Data Mask
    SRAM_WE_N,                                     -- Write Enable
    SRAM_CE_N,                                     -- Chip Enable
    SRAM_OE_N : out std_logic ;                     -- Output Enable

    SW : in std_logic_vector(17 downto 0);         -- DPDT switches
	GPIO_0 : inout std_logic_vector(35 downto 0)
	);
    
end tupacvhdl;

architecture rtl of tupacvhdl is

  component tupacsystem is
  port (
-- 1) global signals:
                 signal clk : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;

              -- the_de2_sram_controller_0
                 signal SRAM_ADDR_from_the_de2_sram_controller_0 : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
                 signal SRAM_CE_N_from_the_de2_sram_controller_0 : OUT STD_LOGIC;
                 signal SRAM_DQ_to_and_from_the_de2_sram_controller_0 : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                 signal SRAM_LB_N_from_the_de2_sram_controller_0 : OUT STD_LOGIC;
                 signal SRAM_OE_N_from_the_de2_sram_controller_0 : OUT STD_LOGIC;
                 signal SRAM_UB_N_from_the_de2_sram_controller_0 : OUT STD_LOGIC;
                 signal SRAM_WE_N_from_the_de2_sram_controller_0 : OUT STD_LOGIC;

              -- the_nes_controller
                 signal data1_to_the_nes_controller : IN STD_LOGIC;
                 signal data2_to_the_nes_controller : IN STD_LOGIC;
                 signal latch1_from_the_nes_controller : OUT STD_LOGIC;
                 signal latch2_from_the_nes_controller : OUT STD_LOGIC;
                 signal pulse1_from_the_nes_controller : OUT STD_LOGIC;
                 signal pulse2_from_the_nes_controller : OUT STD_LOGIC;

              -- the_video_controller
                 signal CLK_25_to_the_video_controller : IN STD_LOGIC;
                 signal VGA_BLANK_from_the_video_controller : OUT STD_LOGIC;
                 signal VGA_B_from_the_video_controller : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                 signal VGA_CLK_from_the_video_controller : OUT STD_LOGIC;
                 signal VGA_G_from_the_video_controller : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                 signal VGA_HS_from_the_video_controller : OUT STD_LOGIC;
                 signal VGA_R_from_the_video_controller : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                 signal VGA_SYNC_from_the_video_controller : OUT STD_LOGIC;
                 signal VGA_VS_from_the_video_controller : OUT STD_LOGIC;
                 signal reset_n_to_the_video_controller : IN STD_LOGIC
  );
  end component;

  signal counter : std_logic_vector(15 downto 0);
  signal reset_n : std_logic;
  signal clk25 : std_logic := '0';
  
begin

  process (CLOCK_50)
  begin
    if CLOCK_50'event and CLOCK_50 = '1' then
      clk25 <= not clk25;
    end if;
  end process;

  process (CLOCK_50)
  begin
    if CLOCK_50'event and CLOCK_50 = '1' then
      if counter = x"ffff" then
        reset_n <= '1';
      else
        reset_n <= '0';
        counter <= counter + 1;
      end if;
    end if;
  end process;

  tupac : tupacsystem port map (
    clk                          => CLOCK_50,
    reset_n                      => reset_n,
    SRAM_ADDR_from_the_de2_sram_controller_0      => SRAM_ADDR,
    SRAM_CE_N_from_the_de2_sram_controller_0      => SRAM_CE_N,
    SRAM_DQ_to_and_from_the_de2_sram_controller_0 => SRAM_DQ,
    SRAM_LB_N_from_the_de2_sram_controller_0      => SRAM_LB_N,
    SRAM_OE_N_from_the_de2_sram_controller_0      => SRAM_OE_N,
    SRAM_UB_N_from_the_de2_sram_controller_0      => SRAM_UB_N,
    SRAM_WE_N_from_the_de2_sram_controller_0      => SRAM_WE_N,

	data1_to_the_nes_controller => GPIO_0(5),
    data2_to_the_nes_controller => GPIO_0(15),
    latch1_from_the_nes_controller => GPIO_0(1),
    latch2_from_the_nes_controller => GPIO_0(11),
    pulse1_from_the_nes_controller => GPIO_0(3),
    pulse2_from_the_nes_controller => GPIO_0(13),

	CLK_25_to_the_video_controller 					  => clk25,
	VGA_BLANK_from_the_video_controller 				  => VGA_BLANK,
    VGA_B_from_the_video_controller 					  => VGA_B,
    VGA_CLK_from_the_video_controller 				  => VGA_CLK,
    VGA_G_from_the_video_controller 					  => VGA_G,
    VGA_HS_from_the_video_controller 					  => VGA_HS,
    VGA_R_from_the_video_controller 					  => VGA_R,
    VGA_SYNC_from_the_video_controller 				  => VGA_SYNC,
	VGA_VS_from_the_video_controller 					  => VGA_VS,
	reset_n_to_the_video_controller			  	      => '0'
  );
  GPIO_0(0) <= CLOCK_50;

end rtl;
